@@ -1,7 +1,7 @@
 /*
  * Load_669.cpp
  * ------------
- * Purpose: 669 Composer / UNIS 669 module loader
+ * Purpose: Composer 669 / UNIS 669 module loader
  * Notes  : <opinion humble="false">This is better than Schism's 669 loader</opinion> :)
  *          (some of this code is "heavily inspired" by Storlek's code from Schism Tracker, and improvements have been made where necessary.)
  * Authors: Olivier Lapicque
@@ -17,10 +17,10 @@ OPENMPT_NAMESPACE_BEGIN
 
 struct _669FileHeader
 {
-	char  magic[2];			// 'if' (0x6669, ha ha) or 'JN'
-	char  songMessage[108];	// Song Message
-	uint8 samples;			// number of samples (1-64)
-	uint8 patterns;			// number of patterns (1-128)
+	char  magic[2];          // 'if' (0x6669, ha ha) or 'JN'
+	char  songMessage[108];  // Song Message
+	uint8 samples;           // number of samples (1-64)
+	uint8 patterns;          // number of patterns (1-128)
 	uint8 restartPos;
 	uint8 orders[128];
 	uint8 tempoList[128];
@@ -50,8 +50,7 @@ struct _669Sample
 		if(mptSmp.nLoopEnd > mptSmp.nLength && mptSmp.nLoopStart == 0)
 		{
 			mptSmp.nLoopEnd = 0;
-		}
-		if(mptSmp.nLoopEnd != 0)
+		} else if(mptSmp.nLoopEnd != 0)
 		{
 			mptSmp.uFlags = CHN_LOOP;
 			mptSmp.SanitizeLoops();
@@ -71,7 +70,13 @@ static bool ValidateHeader(const _669FileHeader &fileHeader)
 	{
 		return false;
 	}
-	for(std::size_t i = 0; i < CountOf(fileHeader.breaks); i++)
+	uint8 invalidCharCount = 0;
+	for(const char c : fileHeader.songMessage)
+	{
+		if(c > 0 && c <= 31 && ++invalidCharCount > 40)
+			return false;
+	}
+	for(std::size_t i = 0; i < std::size(fileHeader.breaks); i++)
 	{
 		if(fileHeader.orders[i] >= 128 && fileHeader.orders[i] < 0xFE)
 			return false;
@@ -144,7 +149,7 @@ bool CSoundFile::Read669(FileReader &file, ModLoadingFlags loadFlags)
 	m_modFormat.formatName = U_("Composer 669");
 	m_modFormat.type = U_("669");
 	m_modFormat.madeWithTracker = !memcmp(fileHeader.magic, "if", 2) ? UL_("Composer 669") : UL_("UNIS 669");
-	m_modFormat.charset = mpt::CharsetCP437;
+	m_modFormat.charset = mpt::Charset::CP437;
 
 	m_nSamples = fileHeader.samples;
 	for(SAMPLEINDEX smp = 1; smp <= m_nSamples; smp++)
@@ -156,13 +161,13 @@ bool CSoundFile::Read669(FileReader &file, ModLoadingFlags loadFlags)
 		if(sampleHeader.length >= 0x4000000)
 			return false;
 		sampleHeader.ConvertToMPT(Samples[smp]);
-		mpt::String::Read<mpt::String::maybeNullTerminated>(m_szNames[smp], sampleHeader.filename);
+		m_szNames[smp] = mpt::String::ReadBuf(mpt::String::maybeNullTerminated, sampleHeader.filename);
 	}
 
 	// Copy first song message line into song title
-	mpt::String::Read<mpt::String::spacePadded>(m_songName, fileHeader.songMessage, 36);
+	m_songName = mpt::String::ReadBuf(mpt::String::spacePadded, fileHeader.songMessage, 36);
 	// Song Message
-	m_songMessage.ReadFixedLineLength(mpt::byte_cast<const mpt::byte*>(fileHeader.songMessage), 108, 36, 0);
+	m_songMessage.ReadFixedLineLength(mpt::byte_cast<const std::byte*>(fileHeader.songMessage), 108, 36, 0);
 
 	// Reading Orders
 	ReadOrderFromArray(Order(), fileHeader.orders, MPT_ARRAY_COUNT(fileHeader.orders), 0xFF, 0xFE);
@@ -186,16 +191,16 @@ bool CSoundFile::Read669(FileReader &file, ModLoadingFlags loadFlags)
 			continue;
 		}
 
-		const ModCommand::COMMAND effTrans[] =
+		static constexpr ModCommand::COMMAND effTrans[] =
 		{
-			CMD_PORTAMENTOUP,	// Slide up (param * 80) Hz on every tick
-			CMD_PORTAMENTODOWN,	// Slide down (param * 80) Hz on every tick
-			CMD_TONEPORTAMENTO,	// Slide to note by (param * 40) Hz on every tick
-			CMD_S3MCMDEX,		// Add (param * 80) Hz to sample frequency
-			CMD_VIBRATO,		// Add (param * 669) Hz on every other tick
-			CMD_SPEED,			// Set ticks per row
-			CMD_PANNINGSLIDE,	// Extended UNIS 669 effect
-			CMD_RETRIG,			// Extended UNIS 669 effect
+			CMD_PORTAMENTOUP,    // Slide up (param * 80) Hz on every tick
+			CMD_PORTAMENTODOWN,  // Slide down (param * 80) Hz on every tick
+			CMD_TONEPORTAMENTO,  // Slide to note by (param * 40) Hz on every tick
+			CMD_S3MCMDEX,        // Add (param * 80) Hz to sample frequency
+			CMD_VIBRATO,         // Add (param * 669) Hz on every other tick
+			CMD_SPEED,           // Set ticks per row
+			CMD_PANNINGSLIDE,    // Extended UNIS 669 effect
+			CMD_RETRIG,          // Extended UNIS 669 effect
 		};
 
 		uint8 effect[8] = { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
@@ -205,29 +210,28 @@ bool CSoundFile::Read669(FileReader &file, ModLoadingFlags loadFlags)
 
 			for(CHANNELINDEX chn = 0; chn < 8; chn++, m++)
 			{
-				uint8 data[3];
-				file.ReadArray(data);
+				const auto [noteInstr, instrVol, effParam] = file.ReadArray<uint8, 3>();
 
-				uint8 note = data[0] >> 2;
-				uint8 instr = ((data[0] & 0x03) << 4) | (data[1] >> 4);
-				uint8 vol = data[1] & 0x0F;
-				if(data[0] < 0xFE)
+				uint8 note = noteInstr >> 2;
+				uint8 instr = ((noteInstr & 0x03) << 4) | (instrVol >> 4);
+				uint8 vol = instrVol & 0x0F;
+				if(noteInstr < 0xFE)
 				{
 					m->note = note + 36 + NOTE_MIN;
 					m->instr = instr + 1;
 					effect[chn] = 0xFF;
 				}
-				if(data[0] <= 0xFE)
+				if(noteInstr <= 0xFE)
 				{
 					m->volcmd = VOLCMD_VOLUME;
 					m->vol = ((vol * 64 + 8) / 15);
 				}
 
-				if(data[2] != 0xFF)
+				if(effParam != 0xFF)
 				{
-					effect[chn] = data[2];
+					effect[chn] = effParam;
 				}
-				if((data[2] & 0x0F) == 0 && data[2] != 0x30)
+				if((effParam & 0x0F) == 0 && effParam != 0x30)
 				{
 					// A param value of 0 resets the effect.
 					effect[chn] = 0xFF;
